/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    :   adslAtucChanIntervalTable_access.c
 * Author       :   Subramani
 * Date         :   17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      :   Amazon
 * Block        :   ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *                  1) Getting the data context pointer & loop context
 *                  pointer
 *                  2) User-defined data access functions (per column) for data
 *                  in table adslAtucChanIntervalTable
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */



/* ===========================================================================
 * Revision History:
 *  Subramani:1) Changed the COLUMN_ADSLATUCCHANINTERVALVALIDDATA return type
 *            from long to int
 *            2) Chnaged the data type of static variable long_ret from long
 *               to u_int32
 * $Log$
 * ===========================================================================
 */

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.access_functions.conf,v 1.6 2003/09/11 22:48:49 rstory Exp $
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
*/

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "adslAtucChanIntervalTable_access.h"
#include "adslAtucChanIntervalTable_enums.h"

/* column number definitions for table adslAtucChanIntervalTable */
#include "adslAtucChanIntervalTable_columns.h"

/*
 * ===========================================================================
 *                          GLOBAL DEFINITIONS
 * ===========================================================================
*/

// *** Global variable definitions
static u_int32 long_ret;

/*
 * ===========================================================================
 *                          LOCAL FUNCTIONS
 * ===========================================================================
*/

/** returns the first data point within the adslAtucChanIntervalTable table
    data.

    Set the my_loop_context variable to the first data point structure
    of your choice (from which you can find the next one).  This could
    be anything from the first node in a linked list, to an integer
    pointer containing the beginning of an array variable.

    Set the my_data_context variable to something to be returned to
    you later that will provide you with the data to return in a given
    row.  This could be the same pointer as what my_loop_context is
    set to, or something different.

    The put_index_data variable contains a list of snmp variable
    bindings, one for each index in your table.  Set the values of
    each appropriately according to the data matching the first row
    and return the put_index_data variable at the end of the function.
*/
netsnmp_variable_list *
adslAtucChanIntervalTable_get_first_data_point(void **my_loop_context,
                                               void **my_data_context,
                                               netsnmp_variable_list *
                                               put_index_data,
                                               netsnmp_iterator_info
                                               *mydata)
{

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-First: adslAtucChanIntervalTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }

    *My_Data_Context=IFTYPE_ADSL_FAST_INTERLEAVE;
    *my_data_context=My_Data_Context;

    put_index_data = adslTable_get_first_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}

/** functionally the same as adslAtucChanIntervalTable_get_first_data_point, but
   my_loop_context has already been set to a previous value and should
   be updated to the next in the list.  For example, if it was a
   linked list, you might want to cast it to your local data type and
   then return my_loop_context->next.  The my_data_context pointer
   should be set to something you need later and the indexes in
   put_index_data updated again. */
netsnmp_variable_list *
adslAtucChanIntervalTable_get_next_data_point(void **my_loop_context,
                                              void **my_data_context,
                                              netsnmp_variable_list *
                                              put_index_data,
                                              netsnmp_iterator_info
                                              *mydata)
{

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-Next: adslAtucChanIntervalTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }
    *My_Data_Context=IFTYPE_ADSL_FAST_INTERLEAVE;
    *my_data_context=My_Data_Context;

     put_index_data = adslTable_get_next_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}


/*
 * User-defined data access functions (per column) for data in table
 * adslAtucChanIntervalTable
 */
/*
 * NOTE:
 * - these get_ routines MUST return data that will not be freed (ie,
 *   use static variables or persistent data).  It will be copied, if
 *   needed, immediately after the get_ routine has been called.
 * - these SET routines must copy the incoming data and can not take
 *   ownership of the memory passed in by the val pointer.
 */

#if defined(HAVE_OPTIMIZED_CODE)

u_int32  *
get_adslAtucChanIntervalData(void *data_context, size_t * ret_len, 
		unsigned int var)
{
  unsigned int flags = 0;
  struct adslAtucChanIntvlInfo adslAtucChanIntvlStruct;
  memset(&adslAtucChanIntvlStruct,0,sizeof(adslAtucChanIntvlInfo));
  adslAtucChanIntvlStruct.ifIndex= (*(int *)data_context);
  adslAtucChanIntvlStruct.IntervalNumber= (*(int *)ret_len);
   
  switch (var) {

    case COLUMN_ADSLATUCCHANINTERVALRECEIVEDBLKS:
  	flags = ATUC_CHAN_INTVL_RECV_BLK_FLAG;
        break;
	
    case COLUMN_ADSLATUCCHANINTERVALTRANSMITTEDBLKS:
	flags = ATUC_CHAN_INTVL_TX_BLK_FLAG;
	break;
	
    case COLUMN_ADSLATUCCHANINTERVALCORRECTEDBLKS:
	flags = ATUC_CHAN_INTVL_CORR_BLK_FLAG;
	break;
	
    case COLUMN_ADSLATUCCHANINTERVALUNCORRECTBLKS:
	flags = ATUC_CHAN_INTVL_UNCORR_BLK_FLAG;
	break;

    default:
        DEBUGMSGTL(("Unknown MIB variable requested!","\n"));
        return NULL;
  }

  SET_FLAG(&adslAtucChanIntvlStruct.flags, flags);

#ifdef ENABLE_DRIVER_CALL
  adsl_ioctl(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO, &adslAtucChanIntvlStruct);
#else
  get_adslAtucChanIntervalTable(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO,
        (void *) &adslAtucChanIntvlStruct);
#endif
  if (IS_FLAG_SET(&adslAtucChanIntvlStruct.flags,flags) == 0) 
    DEBUGMSGTL(("adslMIB-ATUC", "ChanIntervalTable: GET- %x CMV fail\n", flags));

    switch (flags)  {

	 case ATUC_CHAN_INTVL_RECV_BLK_FLAG:
    	    long_ret = adslAtucChanIntvlStruct.chanIntervalRecvdBlks;
            break;
	 
	 case ATUC_CHAN_INTVL_TX_BLK_FLAG:
    	    long_ret = adslAtucChanIntvlStruct.chanIntervalXmitBlks;
	    break;
	  
	 case ATUC_CHAN_INTVL_CORR_BLK_FLAG:
    	    long_ret = adslAtucChanIntvlStruct.chanIntervalCorrectedBlks;
	    break;
	  
	 case ATUC_CHAN_INTVL_UNCORR_BLK_FLAG:
    	    long_ret = adslAtucChanIntvlStruct.chanIntervalUncorrectBlks;
	    break;
	  
         default:
            DEBUGMSGTL(("Unknown MIB variable requested!","\n"));
            return NULL;
     }

    CLR_FLAG(&adslAtucChanIntvlStruct.flags, flags);
    *ret_len = sizeof(u_int32);
    return (u_int32 *) &long_ret;
}

#else

/** XXX:return a data pointer to the data for the
    adslAtucChanIntervalReceivedBlks column and set ret_len to its proper size
    in bytes. */
u_int32  *
get_adslAtucChanIntervalReceivedBlks(void *data_context, size_t * ret_len)
{
    struct adslAtucChanIntvlInfo adslAtucChanIntvlStruct;

    memset(&adslAtucChanIntvlStruct,0,sizeof(adslAtucChanIntvlInfo));
    adslAtucChanIntvlStruct.ifIndex= (*(int *)data_context);
    adslAtucChanIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_RECV_BLK_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO, &adslAtucChanIntvlStruct);
#else
    get_adslAtucChanIntervalTable(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO,
    (void *) &adslAtucChanIntvlStruct);
#endif

  if (IS_FLAG_SET(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_RECV_BLK_FLAG)) 
       DEBUGMSGTL(("adslMIB-ATUC", "ChanIntvlRecvBlk CMV fail\n"));
  
  CLR_FLAG(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_RECV_BLK_FLAG);
  long_ret = adslAtucChanIntvlStruct.chanIntervalRecvdBlks;
  *ret_len = sizeof(adslAtucChanIntvlStruct.chanIntervalRecvdBlks);
  return (u_int32 *) &long_ret;
}

/** XXX:return a data pointer to the data for the
    adslAtucChanIntervalTransmittedBlks column and set
         ret_len to its proper size in bytes. */
u_int32  *
get_adslAtucChanIntervalTransmittedBlks(void *data_context,
                                         size_t * ret_len)
{
    struct adslAtucChanIntvlInfo adslAtucChanIntvlStruct;

    memset(&adslAtucChanIntvlStruct,0,sizeof(adslAtucChanIntvlInfo));
    adslAtucChanIntvlStruct.ifIndex= (*(int *)data_context);
    adslAtucChanIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_TX_BLK_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO, &adslAtucChanIntvlStruct);
#else
    get_adslAtucChanIntervalTable(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO,(void *) &
    adslAtucChanIntvlStruct);
#endif

   if (IS_FLAG_SET(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_TX_BLK_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "ChanIntvlTxBlk CMV fail\n"));

    CLR_FLAG(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_TX_BLK_FLAG);
    long_ret = adslAtucChanIntvlStruct.chanIntervalXmitBlks;
    *ret_len = sizeof(adslAtucChanIntvlStruct.chanIntervalXmitBlks);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the
    adslAtucChanIntervalCorrectedBlks column and set
         ret_len to its proper size in bytes. */
u_int32  *
get_adslAtucChanIntervalCorrectedBlks(void *data_context,
                                       size_t * ret_len)
{
    struct adslAtucChanIntvlInfo adslAtucChanIntvlStruct;

    memset(&adslAtucChanIntvlStruct,0,sizeof(adslAtucChanIntvlInfo));
    adslAtucChanIntvlStruct.ifIndex= (*(int *)data_context);
    adslAtucChanIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_CORR_BLK_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO, &adslAtucChanIntvlStruct);
#else
    get_adslAtucChanIntervalTable(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO,(void *) &
    adslAtucChanIntvlStruct);
#endif

    if (IS_FLAG_SET(&adslAtucChanIntvlStruct.flags,
    	ATUC_CHAN_INTVL_CORR_BLK_FLAG) == 0)
       DEBUGMSGTL(("adslMIB-ATUC", "ChanIntvlCorrBlk CMV fail\n"));


    CLR_FLAG(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_CORR_BLK_FLAG);
    long_ret = adslAtucChanIntvlStruct.chanIntervalCorrectedBlks;
    *ret_len = sizeof(adslAtucChanIntvlStruct.chanIntervalCorrectedBlks);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the
    adslAtucChanIntervalUncorrectBlks column and set
         ret_len to its proper size in bytes. */
u_int32  *
get_adslAtucChanIntervalUncorrectBlks(void *data_context,
                                       size_t * ret_len)
{
    struct adslAtucChanIntvlInfo adslAtucChanIntvlStruct;

    memset(&adslAtucChanIntvlStruct,0,sizeof(adslAtucChanIntvlInfo));
    adslAtucChanIntvlStruct.ifIndex= (*(int *)data_context);
    adslAtucChanIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_UNCORR_BLK_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO, &adslAtucChanIntvlStruct);
#else
    get_adslAtucChanIntervalTable(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO,(void *) &
    adslAtucChanIntvlStruct);
#endif

    if (IS_FLAG_SET(&adslAtucChanIntvlStruct.flags,
    	ATUC_CHAN_INTVL_UNCORR_BLK_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "ChanIntvlUnCorrBlk CMV fail\n"));


    CLR_FLAG(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_UNCORR_BLK_FLAG);
    long_ret = adslAtucChanIntvlStruct.chanIntervalUncorrectBlks;
    *ret_len = sizeof(adslAtucChanIntvlStruct.chanIntervalUncorrectBlks);
    return (u_int32 *) &long_ret;
}

#endif

/** XXX: return a data pointer to the data for the adslAtucChanIntervalValidData
    column and set ret_len to its proper size in bytes. */
int     *
get_adslAtucChanIntervalValidData(void *data_context, size_t * ret_len)
{
    struct adslAtucChanIntvlInfo adslAtucChanIntvlStruct;

    memset(&adslAtucChanIntvlStruct,0,sizeof(adslAtucChanIntvlInfo));
    adslAtucChanIntvlStruct.ifIndex= (*(int *)data_context);
    adslAtucChanIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_VALID_DATA_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO, &adslAtucChanIntvlStruct);
#else
    get_adslAtucChanIntervalTable(Fd, GET_ADSL_ATUC_CHAN_INTVL_INFO,(void *) &
    adslAtucChanIntvlStruct);
#endif

    if (IS_FLAG_SET(&adslAtucChanIntvlStruct.flags,
    	ATUC_CHAN_INTVL_VALID_DATA_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "ChanIntvlValidData CMV fail\n"));

    CLR_FLAG(&adslAtucChanIntvlStruct.flags, ATUC_CHAN_INTVL_VALID_DATA_FLAG);
    long_ret = adslAtucChanIntvlStruct.intervalValidData;
    *ret_len = sizeof(adslAtucChanIntvlStruct.intervalValidData);
    return (int *) &long_ret;
}


#ifndef ENABLE_DRIVER_CALL
int  get_adslAtucChanIntervalTable(int Fd2,int GET_ADSL_atuc_CHAN_INTVL_INFO,
      void *adslAtucChanIntvlTable)
{
    struct adslAtucChanIntvlInfo *adslAtucChanIntvlValue;
    adslAtucChanIntvlValue = (adslAtucChanIntvlInfo *)adslAtucChanIntvlTable;
    adslAtucChanIntvlValue->chanIntervalRecvdBlks = 10;
    adslAtucChanIntvlValue->chanIntervalXmitBlks = 20;
    adslAtucChanIntvlValue->chanIntervalCorrectedBlks = 30;
    adslAtucChanIntvlValue->chanIntervalUncorrectBlks = 5;
    adslAtucChanIntvlValue->intervalValidData = 1;
    return 0;
}
#endif

