/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    :   adslAtucIntervalTable.c
 * Author       :   Subramani
 * Date         :   17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      :   Amazon
 * Block        :   ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *               1) Initializing the adslAtucIntervalTable module
 *               2) Handling requests for the adslAtucIntervalTable table
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */



/* ===========================================================================
 * Revision History:
 * Subramani: Data type for adslAtucIntervalValidData changed from long
 *   to int
 * $Log$
 * ===========================================================================
 */

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.iterate_access.conf,v 1.6 2003/09/29 22:30:36 hardaker Exp $
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
*/

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "adslAtucIntervalTable.h"
#include "adslAtucIntervalTable_access.h"

/*
 * ===========================================================================
 *                          LOCAL FUNCTIONS
 * ===========================================================================
*/

/** Initialize the adslAtucIntervalTable table by defining its contents and how
    it's structured */
void
initialize_table_adslAtucIntervalTable(void)
{
    static oid      adslAtucIntervalTable_oid[] =
        { 1, 3, 6, 1, 2, 1, 10, 94, 1, 1, 8 };
    netsnmp_table_registration_info *table_info;
    netsnmp_handler_registration *my_handler;
    netsnmp_iterator_info *iinfo;

    /** create the table registration information structures */
    table_info = SNMP_MALLOC_TYPEDEF(netsnmp_table_registration_info);
    iinfo = SNMP_MALLOC_TYPEDEF(netsnmp_iterator_info);

    /** if your table is read only, it's easiest to change the
        HANDLER_CAN_RWRITE definition below to HANDLER_CAN_RONLY */
    my_handler =
        netsnmp_create_handler_registration("adslAtucIntervalTable",
                                            adslAtucIntervalTable_handler,
                                            adslAtucIntervalTable_oid,
                                            OID_LENGTH
                                            (adslAtucIntervalTable_oid),
                                            HANDLER_CAN_RONLY);

    if (!my_handler || !table_info || !iinfo) {

#ifdef AMAZON_MEM_LEAK_FIX
         if (my_handler)
            SNMP_FREE(my_handler);
         if (table_info)
            SNMP_FREE(table_info);
          if (iinfo)
            SNMP_FREE(iinfo);
#endif
        snmp_log(LOG_ERR,\
                 "malloc failed in initialize_table_adslAtucIntervalTable");
        return; /** Serious error. */
    }

    /***************************************************
     * Setting up the table's definition
     */
    netsnmp_table_helper_add_indexes(table_info, ASN_INTEGER,
                                               /** index: ifIndex */
                                     ASN_INTEGER,
                                           /** index: adslAtucIntervalNumber */
                                     0);

    /** Define the minimum and maximum accessible columns.  This
        optimizes retrival. */
    table_info->min_column = 2;
    table_info->max_column = 8;

    /** iterator access routines */
    iinfo->get_first_data_point =
        adslAtucIntervalTable_get_first_data_point;
    iinfo->get_next_data_point = adslAtucIntervalTable_get_next_data_point;

    /** you may wish to set these as well */
#ifdef MAYBE_USE_THESE
    iinfo->make_data_context =
        adslAtucIntervalTable_context_convert_function;
    iinfo->free_data_context = adslAtucIntervalTable_data_free;

    /** pick *only* one of these if you use them */
    iinfo->free_loop_context = adslAtucIntervalTable_loop_free;
    iinfo->free_loop_context_at_end = adslAtucIntervalTable_loop_free;
#endif

    /** tie the two structures together */
    iinfo->table_reginfo = table_info;

    /***************************************************
     * registering the table with the master agent
     */
    DEBUGMSGTL(("initialize_table_adslAtucIntervalTable",\
             "Registering table adslAtucIntervalTable as a table iterator\n"));
    netsnmp_register_table_iterator(my_handler, iinfo);
}

/** Initializes the adslAtucIntervalTable module */
void
init_adslAtucIntervalTable(void)
{

  /** here we initialize all the tables we're planning on supporting */
    initialize_table_adslAtucIntervalTable();
}

/** handles requests for the adslAtucIntervalTable table, if anything else
    needs to be done */
int
adslAtucIntervalTable_handler(netsnmp_mib_handler *handler,
                              netsnmp_handler_registration *reginfo,
                              netsnmp_agent_request_info *reqinfo,
                              netsnmp_request_info *requests)
{

    netsnmp_request_info *request;
    netsnmp_table_request_info *table_info;
    netsnmp_variable_list *var;
    long IntervalNum = 1;
    struct commitInfo *ci = NULL;
    void           *data_context = NULL;

 IntervalNum = *(requests->requestvb->name + reginfo->rootoid_len + 3);

#ifdef ENABLE_DRIVER_CALL
    Fd = dev_open(ADSL_DEV_PATH, O_RDONLY);
    if (Fd <=0) {
        snmp_log (LOG_ERR, "Unable to open the device... exiting!\n");
        return SNMP_ERR_GENERR;
    }
#endif

    for (request = requests; request; request = request->next) {
        var = request->requestvb;
        if (request->processed != 0)
            continue;

        switch (reqinfo->mode) {
        case MODE_GET:

            data_context = netsnmp_extract_iterator_context(request);
            if (data_context == NULL) {
                if (reqinfo->mode == MODE_GET) {
                    netsnmp_set_request_error(reqinfo, request,
                                              SNMP_NOSUCHINSTANCE);
                    continue;
                }
            }

        }

        /** extracts the information about the table from the request */
        table_info = netsnmp_extract_table_info(request);
        /** table_info->colnum contains the column number requested */
        /** table_info->indexes contains a linked list of snmp variable
           bindings for the indexes of the table.  Values in the list
           have been set corresponding to the indexes of the
           request */
        if (table_info == NULL) {
            continue;
        }

        switch (reqinfo->mode) {
        case MODE_GET:
            switch (table_info->colnum) {
#if defined(HAVE_OPTIMIZED_CODE)
		    
            case COLUMN_ADSLATUCINTERVALLOFS:
            case COLUMN_ADSLATUCINTERVALLOSS:
            case COLUMN_ADSLATUCINTERVALESS:
            case COLUMN_ADSLATUCINTERVALINITS:
                {
                    u_int32          *retval;
                    size_t          retval_len = IntervalNum;
                    retval =
                        get_adslAtucIntervalData(data_context, &retval_len,
					table_info->colnum);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                            (const u_char *) retval,
                                             retval_len);
                }
                break;
#else
            case COLUMN_ADSLATUCINTERVALLOFS:
                {
                    u_int32          *retval;
                    size_t          retval_len = IntervalNum;
                    retval =
                        get_adslAtucIntervalLofs(data_context,
                                                 &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                            (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCINTERVALLOSS:
                {
                    u_int32          *retval;
                    size_t          retval_len = IntervalNum;
                    retval =
                        get_adslAtucIntervalLoss(data_context,
                                                 &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                            (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCINTERVALESS:
                {
                    u_int32         *retval;
                    size_t          retval_len = IntervalNum;
                    retval =
                        get_adslAtucIntervalESs(data_context, &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                            (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCINTERVALINITS:
                {
                    u_int32         *retval;
                    size_t          retval_len = IntervalNum;
                    retval =
                        get_adslAtucIntervalInits(data_context,
                                                  &retval_len);
                    snmp_set_var_typed_value(var, ASN_GAUGE,
                                            (const u_char *) retval,
                                             retval_len);
                }
                break;
#endif
            case COLUMN_ADSLATUCINTERVALVALIDDATA:
                {
                      int           *retval;
                    size_t          retval_len = IntervalNum;
                    retval =
                        get_adslAtucIntervalValidData(data_context,
                                                      &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            default:
                /** We shouldn't get here */
                snmp_log(LOG_ERR,\
                         "adslAtucIntervalTable_handler: unknown column\n");
            }
            break;

        default:
            snmp_log(LOG_ERR,\
                     "adslAtucIntervalTable_handler: unsupported mode\n");
        }
    }

#ifdef ENABLE_DRIVER_CALL
    dev_close(Fd);
#endif

    return SNMP_ERR_NOERROR;
}
