/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    :   adslLineAlarmConfProfileTable.c
 * Author       :   Subramani
 * Date         :   17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      :   Amazon
 * Block        :   ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *               1) Initializing the adslLineAlarmConfProfileTable module
 *               2) Handling requests for the adslLineAlarmConfProfileTable
 *                  table
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */



/* ===========================================================================
 * Revision History:
 * Subramani: Changed data types as per Rfc-2662
 * $Log$
 * ===========================================================================
*/

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.iterate_access.conf,v 1.6 2003/09/29 22:30:36 hardaker Exp $
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
*/

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "adslLineAlarmConfProfileTable.h"
#include "adslLineAlarmConfProfileTable_checkfns.h"
#include "adslLineAlarmConfProfileTable_access.h"

/*
 * ===========================================================================
 *                          GLOBAL DEFINITIONS
 * ===========================================================================
*/

static netsnmp_oid_stash_node *undoStorage = NULL;
static netsnmp_oid_stash_node *commitStorage = NULL;

/*
 * ===========================================================================
 *                          LOCAL DEFINITIONS
 * ===========================================================================
*/

struct undoInfo {
    void           *ptr;
    size_t          len;
};

struct commitInfo {
    void           *data_context;
    int             have_committed;
    int             new_row;
};

/*
 * ===========================================================================
 *                          LOCAL FUNCTIONS
 * ===========================================================================
*/

void
adslLineAlarmConfProfileTable_free_undoInfo(void *vptr)
{
    struct undoInfo *ui = vptr;
    if (!ui)
        return;
    SNMP_FREE(ui->ptr);
    SNMP_FREE(ui);
}


/** Initialize the adslLineAlarmConfProfileTable table by defining its
    contents and how it's structured */
void
initialize_table_adslLineAlarmConfProfileTable(void)
{
    static oid      adslLineAlarmConfProfileTable_oid[] =
        { 1, 3, 6, 1, 2, 1, 10, 94, 1, 1, 15 };
    netsnmp_table_registration_info *table_info;
    netsnmp_handler_registration *my_handler;
    netsnmp_iterator_info *iinfo;

    /** create the table registration information structures */
    table_info = SNMP_MALLOC_TYPEDEF(netsnmp_table_registration_info);
    iinfo = SNMP_MALLOC_TYPEDEF(netsnmp_iterator_info);

    /** if your table is read only, it's easiest to change the
        HANDLER_CAN_RWRITE definition below to HANDLER_CAN_RONLY */
    my_handler =
        netsnmp_create_handler_registration
        ("adslLineAlarmConfProfileTable",
         adslLineAlarmConfProfileTable_handler,
         adslLineAlarmConfProfileTable_oid,
         OID_LENGTH(adslLineAlarmConfProfileTable_oid),
         HANDLER_CAN_RWRITE);

    if (!my_handler || !table_info || !iinfo) {

#ifdef AMAZON_MEM_LEAK_FIX
         if (my_handler)
            SNMP_FREE(my_handler);
         if (table_info)
            SNMP_FREE(table_info);
          if (iinfo)
            SNMP_FREE(iinfo);
#endif
        snmp_log(LOG_ERR,
                 "malloc failed in\
                 initialize_table_adslLineAlarmConfProfileTable");
        return; /** Serious error. */
    }

    /***************************************************
     * Setting up the table's definition
     */
    netsnmp_table_helper_add_indexes(table_info, ASN_OCTET_STR,
                                    /** index:adslLineAlarmConfProfileName */
                                     0);

    /** Define the minimum and maximum accessible columns.  This
        optimizes retrival. */
    table_info->min_column = 2;
    table_info->max_column = 20;

    /** iterator access routines */
    iinfo->get_first_data_point =
        adslLineAlarmConfProfileTable_get_first_data_point;
    iinfo->get_next_data_point =
        adslLineAlarmConfProfileTable_get_next_data_point;

    /** you may wish to set these as well */
#ifdef MAYBE_USE_THESE
    iinfo->make_data_context =
        adslLineAlarmConfProfileTable_context_convert_function;
    iinfo->free_data_context = adslLineAlarmConfProfileTable_data_free;

    /** pick *only* one of these if you use them */
    iinfo->free_loop_context = adslLineAlarmConfProfileTable_loop_free;
    iinfo->free_loop_context_at_end =
        adslLineAlarmConfProfileTable_loop_free;
#endif

    /** tie the two structures together */
    iinfo->table_reginfo = table_info;

    /***************************************************
     * registering the table with the master agent
     */
    DEBUGMSGTL(("initialize_table_adslLineAlarmConfProfileTable",\
                 "Registering table adslLineAlarmConfProfileTable as a table\
                 iterator\n"));
    netsnmp_register_table_iterator(my_handler, iinfo);
}

/** Initializes the adslLineAlarmConfProfileTable module */
void
init_adslLineAlarmConfProfileTable(void)
{

  /** here we initialize all the tables we're planning on supporting */
    initialize_table_adslLineAlarmConfProfileTable();
}

/** handles requests for the adslLineAlarmConfProfileTable table, if
    anything else needs to be done */
int
adslLineAlarmConfProfileTable_handler(netsnmp_mib_handler *handler,
                                      netsnmp_handler_registration
                                      *reginfo,
                                      netsnmp_agent_request_info *reqinfo,
                                      netsnmp_request_info *requests)
{

    netsnmp_request_info *request;
    netsnmp_table_request_info *table_info;
    netsnmp_variable_list *var;
    struct commitInfo *ci = NULL;

    void           *data_context = NULL;

    oid            *suffix;
    size_t          suffix_len;

    /** column and row index encoded portion */
    suffix = requests->requestvb->name + reginfo->rootoid_len + 1;
    suffix_len = requests->requestvb->name_length -
        (reginfo->rootoid_len + 1);

#ifdef ENABLE_DRIVER_CALL
    Fd = dev_open(ADSL_DEV_PATH, O_RDONLY);
    if (Fd <=0) {
        snmp_log (LOG_ERR, "Unable to open the device... exiting!\n");
        return SNMP_ERR_GENERR;
    }
#endif

    for (request = requests; request; request = request->next) {
        var = request->requestvb;
        if (request->processed != 0)
            continue;

        switch (reqinfo->mode) {
        case MODE_GET:
        case MODE_SET_RESERVE1:
            data_context = netsnmp_extract_iterator_context(request);
            if (data_context == NULL) {
                if (reqinfo->mode == MODE_GET) {
                    netsnmp_set_request_error(reqinfo, request,
                                              SNMP_NOSUCHINSTANCE);
                    continue;
                }
            }
            break;

        default:               /* == the other SET modes */
            ci = netsnmp_oid_stash_get_data(commitStorage,
                                            suffix + 1, suffix_len - 1);
            break;

        }

        /** extracts the information about the table from the request */
        table_info = netsnmp_extract_table_info(request);
        /** table_info->colnum contains the column number requested */
        /** table_info->indexes contains a linked list of snmp variable
           bindings for the indexes of the table.  Values in the list
           have been set corresponding to the indexes of the
           request */
        if (table_info == NULL) {
            continue;
        }

        switch (reqinfo->mode) {
        case MODE_GET:
            switch (table_info->colnum) {
#if defined(HAVE_OPTIMIZED_CODE)
            case COLUMN_ADSLATUCTHRESH15MINLOFS:
            case COLUMN_ADSLATUCTHRESH15MINLOSS:
            case COLUMN_ADSLATUCTHRESH15MINESS:
            case COLUMN_ADSLATURTHRESH15MINLOFS:
            case COLUMN_ADSLATURTHRESH15MINLOSS:
            case COLUMN_ADSLATURTHRESH15MINLPRS:
            case COLUMN_ADSLATURTHRESH15MINESS:
            case COLUMN_ADSLATUCINITFAILURETRAPENABLE:
            case COLUMN_ADSLLINEALARMCONFPROFILEROWSTATUS:
		 {
		    int 	*retval;
		    size_t	retval_len=0;
		    retval = 
			get_adslThresh15MinVal(data_context, &retval_len, table_info->colnum);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
		 }
		 break;

            case COLUMN_ADSLATUCTHRESHFASTRATEUP:
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEUP:
            case COLUMN_ADSLATUCTHRESHFASTRATEDOWN:
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEDOWN:
            case COLUMN_ADSLATURTHRESHFASTRATEUP:
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEUP:
            case COLUMN_ADSLATURTHRESHFASTRATEDOWN:
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEDOWN:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
			get_adslThreshChanVal(data_context, &retval_len, table_info->colnum);
                    snmp_set_var_typed_value(var, ASN_UNSIGNED,
                                             (const u_char *) retval,
                                             retval_len);
		 }
		 break;
#else
            case COLUMN_ADSLATUCTHRESH15MINLOFS:
                {
                      int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucThresh15MinLofs(data_context,
                                                    &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCTHRESH15MINLOSS:
                {
                     int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucThresh15MinLoss(data_context,
                                                    &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;


            case COLUMN_ADSLATUCTHRESH15MINESS:
                {
                      int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucThresh15MinESs(data_context,
                                                   &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCTHRESHFASTRATEUP:
                {
                     u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucThreshFastRateUp(data_context,
                                                     &retval_len);
                    snmp_set_var_typed_value(var, ASN_UNSIGNED,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEUP:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucThreshInterleaveRateUp(data_context,
                                                           &retval_len);
                    snmp_set_var_typed_value(var, ASN_UNSIGNED,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCTHRESHFASTRATEDOWN:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucThreshFastRateDown(data_context,
                                                       &retval_len);
                    snmp_set_var_typed_value(var, ASN_UNSIGNED,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEDOWN:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucThreshInterleaveRateDown(data_context,
                                                             &retval_len);
                    snmp_set_var_typed_value(var, ASN_UNSIGNED,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATUCINITFAILURETRAPENABLE:
                {
                      int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAtucInitFailureTrapEnable(data_context,
                                                          &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATURTHRESH15MINLOFS:
                {
                      int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAturThresh15MinLofs(data_context,
                                                    &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATURTHRESH15MINLOSS:
                {
                      int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAturThresh15MinLoss(data_context,
                                                    &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATURTHRESH15MINLPRS:
                {
                      int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAturThresh15MinLprs(data_context,
                                                    &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATURTHRESH15MINESS:
                {
                     int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAturThresh15MinESs(data_context,
                                                   &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATURTHRESHFASTRATEUP:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAturThreshFastRateUp(data_context,
                                                     &retval_len);
                    snmp_set_var_typed_value(var, ASN_UNSIGNED,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATURTHRESHINTERLEAVERATEUP:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAturThreshInterleaveRateUp(data_context,
                                                           &retval_len);
                    snmp_set_var_typed_value(var, ASN_UNSIGNED,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATURTHRESHFASTRATEDOWN:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAturThreshFastRateDown(data_context,
                                                       &retval_len);
                    snmp_set_var_typed_value(var, ASN_UNSIGNED,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLATURTHRESHINTERLEAVERATEDOWN:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslAturThreshInterleaveRateDown(data_context,
                                                             &retval_len);
                    snmp_set_var_typed_value(var, ASN_UNSIGNED,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

            case COLUMN_ADSLLINEALARMCONFPROFILEROWSTATUS:
                {
                      int           *retval;
                    size_t          retval_len = 0;
                    retval =
                        get_adslLineAlarmConfProfileRowStatus(data_context,
                                                              &retval_len);
                    snmp_set_var_typed_value(var, ASN_INTEGER,
                                             (const u_char *) retval,
                                             retval_len);
                }
                break;

#endif
            default:
                /** We shouldn't get here */
                snmp_log(LOG_ERR,
                    "adslLineAlarmConfProfileTable_handler: unknown column\n");
            }
            break;

        case MODE_SET_RESERVE1:
            ci = netsnmp_oid_stash_get_data(commitStorage,
                                            suffix + 1, suffix_len - 1);

            if (!ci) {
                    /** create the commit storage info */
                ci = SNMP_MALLOC_STRUCT(commitInfo);
                if (!data_context) {
                    ci->data_context =
                        adslLineAlarmConfProfileTable_create_data_context
                        (table_info->indexes);
                    ci->new_row = 1;
                } else {
                    ci->data_context = data_context;
                }
                netsnmp_oid_stash_add_data(&commitStorage,
                                           suffix + 1, suffix_len - 1, ci);
            }
            break;

        case MODE_SET_RESERVE2:
            switch (table_info->colnum) {
#if defined(HAVE_OPTIMIZED_CODE)
            case COLUMN_ADSLATUCTHRESH15MINLOFS:
            case COLUMN_ADSLATUCTHRESH15MINLOSS:
            case COLUMN_ADSLATUCTHRESH15MINESS:
            case COLUMN_ADSLATURTHRESH15MINLOFS:
            case COLUMN_ADSLATURTHRESH15MINLOSS:
            case COLUMN_ADSLATURTHRESH15MINLPRS:
            case COLUMN_ADSLATURTHRESH15MINESS:
            case COLUMN_ADSLATUCINITFAILURETRAPENABLE:
                {
                    int            *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
			get_adslThresh15MinVal(ci->data_context, &retval_len, table_info->colnum);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
		    if (table_info->colnum == COLUMN_ADSLATUCINITFAILURETRAPENABLE)   {
                      ret =
                        check_adslAtucInitFailureTrapEnable(request->
                                                            requestvb->
                                                            type,
                                                            (long *)
                                                            request->
                                                            requestvb->val.
                                                            string,
                                                            request->
                                                            requestvb->
                                                            val_len,
                                                            (long *) retval,
                                                            retval_len);
		    }
		    else { 
			
                    	ret =
                       	  check_adslThresh15MinVal(request->requestvb->
                                                      type,
                                                      (long *) request->
                                                      requestvb->val.
                                                      string,
                                                      request->requestvb->
                                                      val_len, (long *) retval,
                                                      retval_len);
		    }
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
		
            case COLUMN_ADSLATUCTHRESHFASTRATEUP:
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEUP:
            case COLUMN_ADSLATUCTHRESHFASTRATEDOWN:
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEDOWN:
            case COLUMN_ADSLATURTHRESHFASTRATEUP:
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEUP:
            case COLUMN_ADSLATURTHRESHFASTRATEDOWN:
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEDOWN:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
			get_adslThreshChanVal(ci->data_context, &retval_len, table_info->colnum);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslThreshChanVal(request->requestvb->
                                                       type,
                                                       (u_long *) request->
                                                       requestvb->val.
                                                       string,
                                                       request->requestvb->
                                                       val_len, (long *) retval,
                                                       retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
		break;
#else
            case COLUMN_ADSLATUCTHRESH15MINLOFS:
                {
                     int           *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAtucThresh15MinLofs(ci->data_context,
                                                    &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAtucThresh15MinLofs(request->requestvb->
                                                      type,
                                                      (long *) request->
                                                      requestvb->val.
                                                      string,
                                                      request->requestvb->
                                                      val_len, (long *) retval,
                                                      retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATUCTHRESH15MINLOSS:
                {
                    int           *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAtucThresh15MinLoss(ci->data_context,
                                                    &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAtucThresh15MinLoss(request->requestvb->
                                                      type,
                                                      (long *) request->
                                                      requestvb->val.
                                                      string,
                                                      request->requestvb->
                                                      val_len, (long *) retval,
                                                      retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;

            case COLUMN_ADSLATUCTHRESH15MINESS:
                {
                    int           *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAtucThresh15MinESs(ci->data_context,
                                                   &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAtucThresh15MinESs(request->requestvb->
                                                     type,
                                                     (long *) request->
                                                     requestvb->val.string,
                                                     request->requestvb->
                                                     val_len, (long *) retval,
                                                     retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATUCTHRESHFASTRATEUP:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAtucThreshFastRateUp(ci->data_context,
                                                     &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAtucThreshFastRateUp(request->requestvb->
                                                       type,
                                                       (u_long *) request->
                                                       requestvb->val.
                                                       string,
                                                       request->requestvb->
                                                       val_len, (long *) retval,
                                                       retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEUP:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAtucThreshInterleaveRateUp(ci->
                                                           data_context,
                                                           &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAtucThreshInterleaveRateUp(request->
                                                             requestvb->
                                                             type,
                                                             (u_long *)
                                                             request->
                                                             requestvb->
                                                             val.string,
                                                             request->
                                                             requestvb->
                                                             val_len,
                                                            (long *) retval,
                                                             retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATUCTHRESHFASTRATEDOWN:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAtucThreshFastRateDown(ci->data_context,
                                                       &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAtucThreshFastRateDown(request->
                                                         requestvb->type,
                                                         (u_long *)
                                                         request->
                                                         requestvb->val.
                                                         string,
                                                         request->
                                                         requestvb->
                                                         val_len,(long *)retval,
                                                         retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEDOWN:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAtucThreshInterleaveRateDown(ci->
                                                             data_context,
                                                             &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAtucThreshInterleaveRateDown(request->
                                                               requestvb->
                                                               type,
                                                               (u_long *)
                                                               request->
                                                               requestvb->
                                                               val.string,
                                                               request->
                                                               requestvb->
                                                               val_len,
                                                               (long *)retval,
                                                               retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATUCINITFAILURETRAPENABLE:
                {
                    int           *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAtucInitFailureTrapEnable(ci->data_context,
                                                          &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAtucInitFailureTrapEnable(request->
                                                            requestvb->
                                                            type,
                                                            (long *)
                                                            request->
                                                            requestvb->val.
                                                            string,
                                                            request->
                                                            requestvb->
                                                            val_len,
                                                            (long *) retval,
                                                            retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATURTHRESH15MINLOFS:
                {
                    int           *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAturThresh15MinLofs(ci->data_context,
                                                    &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAturThresh15MinLofs(request->requestvb->
                                                      type,
                                                      (long *) request->
                                                      requestvb->val.
                                                      string,
                                                      request->requestvb->
                                                      val_len, (long *) retval,
                                                      retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATURTHRESH15MINLOSS:
                {
                    int           *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAturThresh15MinLoss(ci->data_context,
                                                    &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAturThresh15MinLoss(request->requestvb->
                                                      type,
                                                      (long *) request->
                                                      requestvb->val.
                                                      string,
                                                      request->requestvb->
                                                      val_len, (long *)retval,
                                                      retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATURTHRESH15MINLPRS:
                {
                    int           *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAturThresh15MinLprs(ci->data_context,
                                                    &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAturThresh15MinLprs(request->requestvb->
                                                      type,
                                                      (long *) request->
                                                      requestvb->val.
                                                      string,
                                                      request->requestvb->
                                                      val_len, (long *)retval,
                                                      retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATURTHRESH15MINESS:
                {
                   int           *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAturThresh15MinESs(ci->data_context,
                                                   &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAturThresh15MinESs(request->requestvb->
                                                     type,
                                                     (long *) request->
                                                     requestvb->val.string,
                                                     request->requestvb->
                                                     val_len, (long *)retval,
                                                     retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATURTHRESHFASTRATEUP:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAturThreshFastRateUp(ci->data_context,
                                                     &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAturThreshFastRateUp(request->requestvb->
                                                       type,
                                                       (u_long *) request->
                                                       requestvb->val.
                                                       string,
                                                       request->requestvb->
                                                       val_len, (long *)retval,
                                                       retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEUP:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAturThreshInterleaveRateUp(ci->
                                                           data_context,
                                                           &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAturThreshInterleaveRateUp(request->
                                                             requestvb->
                                                             type,
                                                             (u_long *)
                                                             request->
                                                             requestvb->
                                                             val.string,
                                                             request->
                                                             requestvb->
                                                             val_len,
                                                             (long *)retval,
                                                             retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATURTHRESHFASTRATEDOWN:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAturThreshFastRateDown(ci->data_context,
                                                       &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAturThreshFastRateDown(request->
                                                         requestvb->type,
                                                         (u_long *)
                                                         request->
                                                         requestvb->val.
                                                         string,
                                                         request->
                                                         requestvb->
                                                        val_len, (long *)retval,
                                                         retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEDOWN:
                {
                    u_int32         *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    /** first, get the old value */
                    retval =
                        get_adslAturThreshInterleaveRateDown(ci->
                                                             data_context,
                                                             &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    /** check the new value, possibly against the
                        older value for a valid state transition */
                    ret =
                        check_adslAturThreshInterleaveRateDown(request->
                                                               requestvb->
                                                               type,
                                                               (u_long *)
                                                               request->
                                                               requestvb->
                                                               val.string,
                                                               request->
                                                               requestvb->
                                                               val_len,
                                                               (long *)retval,
                                                               retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        /** remember information for undo purposes later */
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
          // *** This part is commented since rowstatus can't
          // *** be modified by the user for static profiles

       /*     case COLUMN_ADSLLINEALARMCONFPROFILEROWSTATUS:
                {
                    long           *retval;
                    size_t          retval_len = 0;
                    struct undoInfo *ui = NULL;
                    int             ret;

                    //** first, get the old value
                    retval =
                        get_adslLineAlarmConfProfileRowStatus(ci->
                                                              data_context,
                                                              &retval_len);
                    if (retval) {
                        ui = SNMP_MALLOC_STRUCT(undoInfo);
                        ui->len = retval_len;
                        memdup((u_char **) & ui->ptr,
                               (u_char *) retval, ui->len);
                    }

                    //** check the new value, possibly against the
                    //    older value for a valid state transition
                    ret =
                        check_adslLineAlarmConfProfileRowStatus(request->
                                                                requestvb->
                                                                type,
                                                                (long *)
                                                                request->
                                                                requestvb->
                                                                val.string,
                                                                request->
                                                                requestvb->
                                                                val_len,
                                                                retval,
                                                                retval_len);
                    if (ret != 0) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                        adslLineAlarmConfProfileTable_free_undoInfo(ui);
                    } else if (ui) {
                        //** remember information for undo purposes later
                        netsnmp_oid_stash_add_data(&undoStorage,
                                                   suffix, suffix_len, ui);
                    }

                }
                break;
          */
#endif
            default:
                netsnmp_set_request_error(reqinfo, request,
                                          SNMP_ERR_NOTWRITABLE);
                break;
            }
            break;

        case MODE_SET_UNDO:
            /** save a variable copy */
            switch (table_info->colnum) {
#if defined(HAVE_OPTIMIZED_CODE)
            case COLUMN_ADSLATUCTHRESH15MINLOFS:
            case COLUMN_ADSLATUCTHRESH15MINLOSS:
            case COLUMN_ADSLATUCTHRESH15MINESS:
            case COLUMN_ADSLATURTHRESH15MINLOFS:
            case COLUMN_ADSLATURTHRESH15MINLOSS:
            case COLUMN_ADSLATURTHRESH15MINLPRS:
            case COLUMN_ADSLATURTHRESH15MINESS:
            case COLUMN_ADSLATUCINITFAILURETRAPENABLE:
                {
                    int  retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                    	set_adslThresh15MinVal(ci->data_context,
                                       ui->ptr, ui->len, table_info->colnum);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
		break;

            case COLUMN_ADSLATUCTHRESHFASTRATEUP:
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEUP:
            case COLUMN_ADSLATUCTHRESHFASTRATEDOWN:
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEDOWN:
            case COLUMN_ADSLATURTHRESHFASTRATEUP:
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEUP:
            case COLUMN_ADSLATURTHRESHFASTRATEDOWN:
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEDOWN:
                {
                    int  retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                    	set_adslThreshChanVal(ci->data_context,
                                       ui->ptr, ui->len, table_info->colnum);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
#else
            case COLUMN_ADSLATUCTHRESH15MINLOFS:
                {
                    int             ret;
                    ret = set_adslAtucThresh15MinLofs(ci->data_context,
                                                      (long *) request->
                                                      requestvb->val.
                                                      string,
                                                      request->requestvb->
                                                      val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATUCTHRESH15MINLOSS:
                {
                    int             ret;
                    ret = set_adslAtucThresh15MinLoss(ci->data_context,
                                                      (long *) request->
                                                      requestvb->val.
                                                      string,
                                                      request->requestvb->
                                                      val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATUCTHRESH15MINESS:
                {
                    int             ret;
                    ret = set_adslAtucThresh15MinESs(ci->data_context,
                                                     (long *) request->
                                                     requestvb->val.string,
                                                     request->requestvb->
                                                     val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATUCTHRESHFASTRATEUP:
                {
                    int             ret;
                    ret = set_adslAtucThreshFastRateUp(ci->data_context,
                                                       (u_long *) request->
                                                       requestvb->val.
                                                       string,
                                                       request->requestvb->
                                                       val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEUP:
                {
                    int             ret;
                    ret =
                        set_adslAtucThreshInterleaveRateUp(ci->
                                                           data_context,
                                                           (u_long *)
                                                           request->
                                                           requestvb->val.
                                                           string,
                                                           request->
                                                           requestvb->
                                                           val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATUCTHRESHFASTRATEDOWN:
                {
                    int             ret;
                    ret = set_adslAtucThreshFastRateDown(ci->data_context,
                                                         (u_long *)
                                                         request->
                                                         requestvb->val.
                                                         string,
                                                         request->
                                                         requestvb->
                                                         val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEDOWN:
                {
                    int             ret;
                    ret =
                        set_adslAtucThreshInterleaveRateDown(ci->
                                                             data_context,
                                                             (u_long *)
                                                             request->
                                                             requestvb->
                                                             val.string,
                                                             request->
                                                             requestvb->
                                                             val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATUCINITFAILURETRAPENABLE:
                {
                    int             ret;
                    ret =
                        set_adslAtucInitFailureTrapEnable(ci->data_context,
                                                          (long *)
                                                          request->
                                                          requestvb->val.
                                                          string,
                                                          request->
                                                          requestvb->
                                                          val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESH15MINLOFS:
                {
                    int             ret;
                    ret = set_adslAturThresh15MinLofs(ci->data_context,
                                                      (long *) request->
                                                      requestvb->val.
                                                      string,
                                                      request->requestvb->
                                                      val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESH15MINLOSS:
                {
                    int             ret;
                    ret = set_adslAturThresh15MinLoss(ci->data_context,
                                                      (long *) request->
                                                      requestvb->val.
                                                      string,
                                                      request->requestvb->
                                                      val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESH15MINLPRS:
                {
                    int             ret;
                    ret = set_adslAturThresh15MinLprs(ci->data_context,
                                                      (long *) request->
                                                      requestvb->val.
                                                      string,
                                                      request->requestvb->
                                                      val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESH15MINESS:
                {
                    int             ret;
                    ret = set_adslAturThresh15MinESs(ci->data_context,
                                                     (long *) request->
                                                     requestvb->val.string,
                                                     request->requestvb->
                                                     val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESHFASTRATEUP:
                {
                    int             ret;
                    ret = set_adslAturThreshFastRateUp(ci->data_context,
                                                       (u_long *) request->
                                                       requestvb->val.
                                                       string,
                                                       request->requestvb->
                                                       val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEUP:
                {
                    int             ret;
                    ret =
                        set_adslAturThreshInterleaveRateUp(ci->
                                                           data_context,
                                                           (u_long *)
                                                           request->
                                                           requestvb->val.
                                                           string,
                                                           request->
                                                           requestvb->
                                                           val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESHFASTRATEDOWN:
                {
                    int             ret;
                    ret = set_adslAturThreshFastRateDown(ci->data_context,
                                                         (u_long *)
                                                         request->
                                                         requestvb->val.
                                                         string,
                                                         request->
                                                         requestvb->
                                                         val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEDOWN:
                {
                    int             ret;
                    ret =
                        set_adslAturThreshInterleaveRateDown(ci->
                                                             data_context,
                                                             (u_long *)
                                                             request->
                                                             requestvb->
                                                             val.string,
                                                             request->
                                                             requestvb->
                                                             val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
        // *** This part is commented since rowstatus can't
      // *** be modified by the user for static profiles

        /*    case COLUMN_ADSLLINEALARMCONFPROFILEROWSTATUS:
                {
                    int             ret;
                    ret =
                        set_adslLineAlarmConfProfileRowStatus(ci->
                                                              data_context,
                                                              (long *)
                                                              request->
                                                              requestvb->
                                                              val.string,
                                                              request->
                                                              requestvb->
                                                              val_len);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                    if (*request->requestvb->val.integer == RS_DESTROY) {
                        ci->new_row = -1;
                    }
                }
                break;
        */
#endif
	default:
            netsnmp_set_request_error(reqinfo, request,
                                          SNMP_ERR_NOTWRITABLE);
                break;
		
            }
            break;

        case MODE_SET_COMMIT:
            if (!ci->have_committed) {
                    /** do this once per row only */
                adslLineAlarmConfProfileTable_commit_row(&ci->data_context,
                                                         ci->new_row);
                ci->have_committed = 1;
            }
            break;

        case MODE_SET_ACTION:
             /** save a variable copy */
            switch (table_info->colnum) {
#if defined(HAVE_OPTIMIZED_CODE)
            case COLUMN_ADSLATUCTHRESH15MINLOFS:
            case COLUMN_ADSLATUCTHRESH15MINLOSS:
            case COLUMN_ADSLATUCTHRESH15MINESS:
            case COLUMN_ADSLATURTHRESH15MINLOFS:
            case COLUMN_ADSLATURTHRESH15MINLOSS:
            case COLUMN_ADSLATURTHRESH15MINLPRS:
            case COLUMN_ADSLATURTHRESH15MINESS:
            case COLUMN_ADSLATUCINITFAILURETRAPENABLE:
                {
                    int   ret;
                    ret = set_adslThresh15MinVal(ci->data_context,
                                            (long *) request->requestvb->val.string,
                                             request->requestvb->val_len, table_info->colnum);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
		break;

            case COLUMN_ADSLATUCTHRESHFASTRATEUP:
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEUP:
            case COLUMN_ADSLATUCTHRESHFASTRATEDOWN:
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEDOWN:
            case COLUMN_ADSLATURTHRESHFASTRATEUP:
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEUP:
            case COLUMN_ADSLATURTHRESHFASTRATEDOWN:
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEDOWN:
                {
                    int  ret;
                    ret = set_adslThreshChanVal(ci->data_context,
                                            (long *) request->requestvb->val.string,
                                             request->requestvb->val_len, table_info->colnum);
                    if (ret) {
                        netsnmp_set_request_error(reqinfo, request, ret);
                    }
                }
                break;
#else
            case COLUMN_ADSLATUCTHRESH15MINLOFS:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAtucThresh15MinLofs(ci->data_context,
                                                    ui->ptr, ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATUCTHRESH15MINLOSS:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAtucThresh15MinLoss(ci->data_context,
                                                    ui->ptr, ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;

            case COLUMN_ADSLATUCTHRESH15MINESS:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAtucThresh15MinESs(ci->data_context,
                                                   ui->ptr, ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATUCTHRESHFASTRATEUP:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAtucThreshFastRateUp(ci->data_context,
                                                     ui->ptr, ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEUP:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAtucThreshInterleaveRateUp(ci->
                                                           data_context,
                                                           ui->ptr,
                                                           ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATUCTHRESHFASTRATEDOWN:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAtucThreshFastRateDown(ci->data_context,
                                                       ui->ptr, ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATUCTHRESHINTERLEAVERATEDOWN:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAtucThreshInterleaveRateDown(ci->
                                                             data_context,
                                                             ui->ptr,
                                                             ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATUCINITFAILURETRAPENABLE:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAtucInitFailureTrapEnable(ci->data_context,
                                                          ui->ptr,
                                                          ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESH15MINLOFS:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAturThresh15MinLofs(ci->data_context,
                                                    ui->ptr, ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESH15MINLOSS:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAturThresh15MinLoss(ci->data_context,
                                                    ui->ptr, ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESH15MINLPRS:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAturThresh15MinLprs(ci->data_context,
                                                    ui->ptr, ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESH15MINESS:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAturThresh15MinESs(ci->data_context,
                                                   ui->ptr, ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESHFASTRATEUP:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAturThreshFastRateUp(ci->data_context,
                                                     ui->ptr, ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEUP:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAturThreshInterleaveRateUp(ci->
                                                           data_context,
                                                           ui->ptr,
                                                           ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESHFASTRATEDOWN:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAturThreshFastRateDown(ci->data_context,
                                                       ui->ptr, ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
            case COLUMN_ADSLATURTHRESHINTERLEAVERATEDOWN:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslAturThreshInterleaveRateDown(ci->
                                                             data_context,
                                                             ui->ptr,
                                                             ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
      // *** This part is commented since rowstatus can't
      // *** be modified by the user for static profiles

     /*       case COLUMN_ADSLLINEALARMCONFPROFILEROWSTATUS:
                {
                    int             retval;
                    struct undoInfo *ui;
                    ui = netsnmp_oid_stash_get_data(undoStorage,
                                                    suffix, suffix_len);
                    retval =
                        set_adslLineAlarmConfProfileRowStatus(ci->
                                                              data_context,
                                                              ui->ptr,
                                                              ui->len);
                    if (retval) {
                        netsnmp_set_request_error(reqinfo, request,
                                                  SNMP_ERR_UNDOFAILED);
                    }
                }
                break;
      */
#endif
            default:
                netsnmp_set_request_error(reqinfo, request,
                                          SNMP_ERR_NOTWRITABLE);
                break;
            }
            break;

        case MODE_SET_FREE:
            break;

        default:
            snmp_log(LOG_ERR,\
                "adslLineAlarmConfProfileTable_handler: unsupported mode\n");
        }
    }

    /** clean up after all requset processing has ended */
    switch (reqinfo->mode) {
    case MODE_SET_UNDO:
    case MODE_SET_FREE:
    case MODE_SET_COMMIT:
        /** clear out the undo cache */
        netsnmp_oid_stash_free(&undoStorage,
                               adslLineAlarmConfProfileTable_free_undoInfo);
        netsnmp_oid_stash_free(&commitStorage, netsnmp_oid_stash_no_free);
    }

#ifdef ENABLE_DRIVER_CALL
    dev_close(Fd);
#endif

    return SNMP_ERR_NOERROR;
}
