/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    : adslTable_access.c
 * Author       : Subbi
 * Date         : 17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      : Amazon
 * Block        : ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *                 1) Getting the data context pointer & loop context pointer
 *                 2) Return the IfType of an IfEntry in the IfTable
 *                 3) Return the Number of interfaces in the IfTable
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */

/* ===========================================================================
 * Revision History:
 *
 * $Log$
 * ===========================================================================
 */

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.access_functions.conf,v 1.6 2003/09/11 22:48:49 rstory Exp $
 */

/************************* Important Points *************/
/* There will be two interfaces registered in the interface table of
*  Kernel by adslDriver one as interface type "adsl" other as "Fast"
*  We will be always getting the IfIndex based on interface name
*  This logic is not suitable for multiple adsl Interfaces
*/

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
 */

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "adslTable_access.h"
#include <linux/socket.h>
#include <linux/if.h>
#include <linux/sockios.h>

/*
 * ===========================================================================
 *                           GLOBAL DEFINITIONS
 * ===========================================================================
 */
// *** Global variable definitions

/*
 * ===========================================================================
 *                           EXPORTED FUNCTIONS
 * ===========================================================================
 */

/* This is used, because the ADSL-MIB augments ifTable */
extern unsigned char *var_ifEntry(struct variable *,
                                  oid *, size_t *,
                                  int, size_t *, WriteMethod **);
extern  unsigned char *var_interfaces(struct variable * ,
                                oid * ,size_t * ,
                                int , size_t * , WriteMethod ** );


/*
 * ===========================================================================
 *                           GLOBAL FUNCTIONS
 * ===========================================================================
 */
/** returns the first data point within the adslLineTable table data.

    Set the my_loop_context variable to the first data point structure
    of your choice (from which you can find the next one).  This could
    be anything from the first node in a linked list, to an integer
    pointer containing the beginning of an array variable.

    Set the my_data_context variable to something to be returned to
    you later that will provide you with the data to return in a given
    row.  This could be the same pointer as what my_loop_context is
    set to, or something different.

    The put_index_data variable contains a list of snmp variable
    bindings, one for each index in your table.  Set the values of
    each appropriately according to the data matching the first row
    and return the put_index_data variable at the end of the function.
*/
netsnmp_variable_list *
adslTable_get_first_data_point(void **my_loop_context,
                                   void **my_data_context,
                                   netsnmp_variable_list *put_index_data,
                                   netsnmp_iterator_info *mydata)
{

/*  Find the Total No. of entries in IfTable
    Find the first adsl Entry in the IfTable
    Update loop_context,data_context and put_index_data variables
*/


    int i , p ,interface_count,incr_position;
	int adsl_if_available = 0;

    netsnmp_variable_list *vptr;
    interface_count = i = p = 0;

     if (!position) {
        position = SNMP_MALLOC_TYPEDEF(int);
        if (!position)
            return NULL;
     }

     if (!Data_Context) {
        Data_Context = SNMP_MALLOC_TYPEDEF(int);
        if (!Data_Context) {
           SNMP_FREE(position);
           return NULL;
	}
     }

    interface_count = GetNoInterfaces();
    *position = 0;
    if(!interface_count) {
        SNMP_FREE(position);
        SNMP_FREE(Data_Context);
        return NULL;
    }

    // Check wether Interfaces start with 0 or 1
    for (i=1;i<= interface_count;i++) {
         p = GetType(i);
         if (p != 0 && (p == (*(int *)(*my_data_context))))  {
			    adsl_if_available = 1;
                *Data_Context = i;
                *my_loop_context = position;
                *my_data_context = Data_Context;
                 break;
          } 
	  else if (p != 0 && (IFTYPE_ADSL_FAST_INTERLEAVE ==
                             (*(int *)(*my_data_context))))   {
               	if(p == IFTYPE_ADSL_INTERLEAVE ||
                       	p == IFTYPE_ADSL_FAST )  {
			        adsl_if_available = 1;
                  	*Data_Context = i;
                  	*my_loop_context = position;
                  	*my_data_context = Data_Context;
                     break;
                }
          }
    }

	if (!adsl_if_available)
	 return NULL;

    vptr = put_index_data;
    snmp_set_var_value(vptr, (u_char *) Data_Context ,sizeof (*Data_Context));
    return put_index_data;
}


/* For adslLineAlarmConfProfileTable to get first data point */
netsnmp_variable_list *
adslAlarmTable_get_first_data_point(void **my_loop_context,
                                   void **my_data_context,
                                   netsnmp_variable_list *put_index_data,
                                   netsnmp_iterator_info *mydata)
{

/*  Find the Total No. of entries in IfTable
    Find the first adsl Entry in the IfTable
    Update loop_context,data_context and put_index_data variables
*/


    int i , p ,interface_count,incr_position;
	int adsl_if_available = 0;

    netsnmp_variable_list *vptr;
    interface_count = i = p = 0;

    if (!position) {
        position = SNMP_MALLOC_TYPEDEF(int);
        if (!position)
          return NULL;
    }
    if (!Alarm_Data_Context) {
        Alarm_Data_Context = malloc(ALARM_PROF_NAME_STR_LENGTH);
        if (!Alarm_Data_Context) {
             SNMP_FREE(position);
             return NULL;
	}
    }
    interface_count = GetNoInterfaces();
    *position = 0;
    if(!interface_count) {
        SNMP_FREE(position);
        SNMP_FREE(Alarm_Data_Context);
        return NULL;
    }

    // Check wether Interfaces start with 0 or 1
    for (i=1;i<= interface_count;i++) {
         p = GetType(i);
         if (p != 0 && (p == (*(int *)(*my_data_context)))) {
			   adsl_if_available = 1;
               sprintf(Alarm_Data_Context, "%010d", i);
               *my_loop_context = position;
               *my_data_context = Alarm_Data_Context;
                 break;
          }
	  else if (p != 0 && (IFTYPE_ADSL_FAST_INTERLEAVE ==
                                (*(int *)(*my_data_context)))) {
                  if(p == IFTYPE_ADSL_INTERLEAVE ||
                        p == IFTYPE_ADSL_FAST) {
			  		  adsl_if_available = 1;
                      sprintf(Alarm_Data_Context, "%010d", i);
                      *my_loop_context = position;
                      *my_data_context = Alarm_Data_Context;
                      break;
                  }
          }
   }
	
	if (!adsl_if_available)
	 return NULL;

   vptr = put_index_data;
   snmp_set_var_value(vptr, (u_char *) Alarm_Data_Context , strlen(Alarm_Data_Context));
   return put_index_data;
}


/** functionally the same as adslLineTable_get_first_data_point, but
   my_loop_context has already been set to a previous value and should
   be updated to the next in the list.  For example, if it was a
   linked list, you might want to cast it to your local data type and
   then return my_loop_context->next.  The my_data_context pointer
   should be set to something you need later and the indexes in
   put_index_data updated again. */
netsnmp_variable_list *
adslTable_get_next_data_point(void **my_loop_context,
                                  void **my_data_context,
                                  netsnmp_variable_list *put_index_data,
                                  netsnmp_iterator_info *mydata)
{

/* Go the data point where the my_loop_context pointer points to
   Search for the next adsl entry in the IfTable
   Update the put_index_data,loop_context and data_context pointers
*/



    int interface_count , i , p , search_for_nextentry ;
	int adsl_if_available = 0;

    netsnmp_variable_list *vptr;
    interface_count = i = p = search_for_nextentry = 0;

    if (!Data_Context) {
        Data_Context = SNMP_MALLOC_TYPEDEF(int);
        if (!Data_Context)
             return NULL;
    }

    position=(int *)(*my_loop_context);
    if (position == NULL) {
        position = SNMP_MALLOC_TYPEDEF(int);
        if (!position) {
           SNMP_FREE(Data_Context);
           return NULL;
        }
    }

    interface_count = GetNoInterfaces();
    if(!interface_count) {
        SNMP_FREE(position);
        SNMP_FREE(Data_Context);
        return NULL;
     }

     // Check wether Interfaces start with 0 or 1
     for (i=1;i<= interface_count;i++) {
         p = GetType(i); //GetType

         if (p != 0 && (IFTYPE_ADSL_FAST_INTERLEAVE ==
                (*(int *)(*my_data_context))) &&
                 (search_for_nextentry==0) &&
                  (*position == (*(int *)(*my_loop_context))))  {

               if (p == IFTYPE_ADSL_INTERLEAVE || p == IFTYPE_ADSL_FAST)  {

                   /* The following three lines of code were added to handle 
		    * the case if there is only one interface in the box
		    */
			        adsl_if_available = 1;
                    *Data_Context = i;
                    *my_loop_context = position;
                    *my_data_context = Data_Context;
                    search_for_nextentry =1;
                    continue;
               }
           }
	   else if (p != 0 && (IFTYPE_ADSL_FAST_INTERLEAVE ==
                  (*(int *)(*my_data_context))) && (search_for_nextentry==1)) {

               if (p == IFTYPE_ADSL_INTERLEAVE || 
			p == IFTYPE_ADSL_FAST) {
			        adsl_if_available = 1;
               	    *Data_Context = i;
               	    *my_loop_context = position;
               	    *my_data_context = Data_Context;
               	    vptr = put_index_data;
               	    snmp_set_var_value(vptr, (u_char *) Data_Context ,
               	    sizeof (*Data_Context));
               	    return put_index_data;
              	}
           }

           if (p != 0 && (p == (*(int *)(*my_data_context))) &&
                               (search_for_nextentry==0) &&
                             (*position == (*(int *)(*my_loop_context))))   {

         // The following three lines of code were added to handle the case
         // if there is only one interface in the box
			    adsl_if_available = 1;
                *Data_Context = i;
                *my_loop_context = position;
                *my_data_context = Data_Context;
                 search_for_nextentry =1;
                 continue;
           } 
	   else if (p != 0 && (p == (*(int *)(*my_data_context)))
                           && (search_for_nextentry==1)) {
			    adsl_if_available = 1;
                *Data_Context = i;
                *my_loop_context = position;
                *my_data_context = Data_Context;
           }
           (*position)++;
     }

     if (*position > interface_count) {
        SNMP_FREE(position);
        SNMP_FREE(Data_Context);
        *my_loop_context = NULL;
        *my_data_context = NULL;
        return NULL;
     }

	if (!adsl_if_available)
	 return NULL;

     vptr = put_index_data;
     snmp_set_var_value(vptr, (u_char *) Data_Context ,sizeof (*Data_Context));
     return put_index_data;
}


/* For adslLineAlarmConfProfileTable to get next data point */

netsnmp_variable_list *
adslAlarmTable_get_next_data_point(void **my_loop_context,
                                  void **my_data_context,
                                  netsnmp_variable_list *put_index_data,
                                  netsnmp_iterator_info *mydata)
{

/* Go the data point where the my_loop_context pointer points to
   Search for the next adsl entry in the IfTable
   Update the put_index_data,loop_context and data_context pointers
*/



    int interface_count , i , p , search_for_nextentry ;
	int adsl_if_available = 0;

    netsnmp_variable_list *vptr;
    interface_count = i = p = search_for_nextentry = 0;


    if (!Alarm_Data_Context) {

        Alarm_Data_Context = malloc(ALARM_PROF_NAME_STR_LENGTH);
        if (!Alarm_Data_Context)
           return NULL;
    }

    position=(int *)(*my_loop_context);
    if (position == NULL)
    {
        position = SNMP_MALLOC_TYPEDEF(int);
        if (!position) {
           SNMP_FREE(Alarm_Data_Context);
           return NULL;
        }
    }

    interface_count = GetNoInterfaces();
    if(!interface_count)  {
        SNMP_FREE(position);
        SNMP_FREE(Alarm_Data_Context);
        return NULL;
    }

     // Check wether Interfaces start with 0 or 1
    for (i=1;i<= interface_count;i++) {
         p = GetType(i); //GetType

         if (p != 0 && (IFTYPE_ADSL_FAST_INTERLEAVE == (*(int *)
                        (*my_data_context))) &&
                        (search_for_nextentry==0) &&
                        (*position == (*(int *)(*my_loop_context))))  {

               if (p == IFTYPE_ADSL_INTERLEAVE || p == IFTYPE_ADSL_FAST) {

             // The following three lines of code were added to handle the case
             // if there is only one interface in the box


			    adsl_if_available = 1;
                sprintf(Alarm_Data_Context, "%010d", i);
                *my_loop_context = position;
                *my_data_context = Alarm_Data_Context;
                 search_for_nextentry =1;
                 continue;
               }
           } 
	   else if (p != 0 && (IFTYPE_ADSL_FAST_INTERLEAVE ==
                  (*(int *)(*my_data_context))) && (search_for_nextentry==1)) {

  	     if (p == IFTYPE_ADSL_INTERLEAVE || p == IFTYPE_ADSL_FAST) {

			    adsl_if_available = 1;
                 sprintf(Alarm_Data_Context, "%010d", i);
                *my_loop_context = position;
                *my_data_context = Alarm_Data_Context;

                 vptr = put_index_data;
                 snmp_set_var_value(vptr, (u_char *) Alarm_Data_Context ,
                 sizeof (*Alarm_Data_Context));
                 return put_index_data;
              }
           }

           if (p != 0 && (p == (*(int *)(*my_data_context))) &&
                            (search_for_nextentry==0) &&
                            (*position == (*(int *)(*my_loop_context))))  {

         // The following three lines of code were added to handle the case
         // if there is only one interface in the box

			    adsl_if_available = 1;
                sprintf(Alarm_Data_Context, "%010d", i);
                *my_loop_context = position;
                *my_data_context = Alarm_Data_Context;
                 search_for_nextentry =1;
                 continue;
            } 
	    else if (p != 0 && (p == (*(int *)(*my_data_context)))
                            && (search_for_nextentry==1)) {

			    adsl_if_available = 1;
                sprintf(Alarm_Data_Context, "%010d", i);
                *my_loop_context = position;
                *my_data_context = Alarm_Data_Context;
           }
           (*position)++;
        }

        if (*position > interface_count) {
           SNMP_FREE(position);
           SNMP_FREE(Alarm_Data_Context);
           *my_loop_context = NULL;
           *my_data_context = NULL;
           return NULL;
        }

	if (!adsl_if_available)
	 return NULL;

        vptr = put_index_data;
        snmp_set_var_value(vptr, (u_char *) Alarm_Data_Context ,strlen(Alarm_Data_Context));
        return put_index_data;
}


int adsl_ioctl(int fd, int cmd, void *structptr)
{
  return (ioctl(fd, cmd, structptr));
}


int dev_open(const char *pathname, int flags)
{
  return (open(pathname, flags));
}


int dev_close(int fd)
{
  return (close(fd));
}


/*** XXX Return the IfType of an IfEntry in the IfTable XXX ***/
static int
GetType(int index)
{
    oid             name[MAX_OID_LEN] = { 1, 3, 6, 1, 2, 1, 2, 2, 1, 3 };
    size_t          length = 10;
    struct variable ifType_variable =
        { 3, ASN_INTEGER, RONLY, var_ifEntry, 10,
        {1, 3, 6, 1, 2, 1, 2, 2, 1, 3}
    };
    unsigned char  *p;
    size_t          var_len;
    WriteMethod    *write_method;

    name[length] = index;
    length++;

    p = var_ifEntry(&ifType_variable,
                    name, &length,
                    1 /* exact */ , &var_len, &write_method);
    if (!p)
        return 0;

    return *(int *) p;
}


/*** XXX Return the Number of interfaces in the IfTable XXX ***/
static int
GetNoInterfaces()
{
    oid             name[MAX_OID_LEN] = { 1, 3, 6, 1, 2, 1, 2, 2, 1, 0 };
    size_t          length = 10;
    struct variable ifType_variable =
        { 0, ASN_INTEGER, RONLY, var_ifEntry, 10,
        {1, 3, 6, 1, 2, 1, 2, 2, 1, 0}
    };
    unsigned char  *p;
    size_t          var_len;
    WriteMethod    *write_method;

    name[length] = 0;
    length++;

    p = var_interfaces(&ifType_variable,
                    name, &length,
                    1 /* exact */ , &var_len, &write_method);
    if (!p)
        return 0;

    return *(int *) p;
}


